@@ -305,15 +305,6 @@ class JobQueueFull(JobQueueError):
   """
 
 
-class ConfdFatalError(GenericError):
-  """A fatal failure in Ganeti confd.
-
-  Events that compromise the ability of confd to proceed further.
-  (for example: inability to load the config file)
-
-  """
-
-
 class ConfdRequestError(GenericError):
   """A request error in Ganeti confd.
 
@@ -345,6 +336,12 @@ class UdpDataSizeError(GenericError):
   """
 
 
+class NoCtypesError(GenericError):
+  """python ctypes module is not found in the system.
+
+  """
+
+
 # errors should be added above
 
 
