@@ -64,6 +64,9 @@ REQ_SET_WATCHER_PAUSE = "SetWatcherPause"
 DEF_CTMO = 10
 DEF_RWTO = 60
 
+# WaitForJobChange timeout
+WFJC_TIMEOUT = (DEF_RWTO - 1) / 2
+
 
 class ProtocolError(Exception):
   """Denotes an error in the server communication"""
@@ -119,15 +122,12 @@ class Transport:
 
   """
 
-  def __init__(self, address, timeouts=None, eom=None):
+  def __init__(self, address, timeouts=None):
     """Constructor for the Client class.
 
     Arguments:
       - address: a valid address the the used transport class
       - timeout: a list of timeouts, to be used on connect and read/write
-      - eom: an identifier to be used as end-of-message which the
-        upper-layer will guarantee that this identifier will not appear
-        in any message
 
     There are two timeouts used since we might want to wait for a long
     time for a response, but the connect timeout should be lower.
@@ -150,11 +150,6 @@ class Transport:
     self._buffer = ""
     self._msgs = collections.deque()
 
-    if eom is None:
-      self.eom = '\3'
-    else:
-      self.eom = eom
-
     try:
       self.socket = socket.socket(socket.AF_UNIX, socket.SOCK_STREAM)
 
@@ -200,12 +195,12 @@ class Transport:
     This just sends a message and doesn't wait for the response.
 
     """
-    if self.eom in msg:
+    if constants.LUXI_EOM in msg:
       raise EncodingError("Message terminator found in payload")
     self._CheckSocket()
     try:
       # TODO: sendall is not guaranteed to send everything
-      self.socket.sendall(msg + self.eom)
+      self.socket.sendall(msg + constants.LUXI_EOM)
     except socket.timeout, err:
       raise TimeoutError("Sending timeout: %s" % str(err))
 
@@ -235,7 +230,7 @@ class Transport:
         break
       if not data:
         raise ConnectionClosedError("Connection closed while reading")
-      new_msgs = (self._buffer + data).split(self.eom)
+      new_msgs = (self._buffer + data).split(constants.LUXI_EOM)
       self._buffer = new_msgs.pop()
       self._msgs.extend(new_msgs)
     return self._msgs.popleft()
@@ -373,11 +368,27 @@ class Client(object):
     return self.CallMethod(REQ_AUTOARCHIVE_JOBS, (age, timeout))
 
   def WaitForJobChangeOnce(self, job_id, fields,
-                           prev_job_info, prev_log_serial):
-    timeout = (DEF_RWTO - 1) / 2
+                           prev_job_info, prev_log_serial,
+                           timeout=WFJC_TIMEOUT):
+    """Waits for changes on a job.
+
+    @param job_id: Job ID
+    @type fields: list
+    @param fields: List of field names to be observed
+    @type prev_job_info: None or list
+    @param prev_job_info: Previously received job information
+    @type prev_log_serial: None or int/long
+    @param prev_log_serial: Highest log serial number previously received
+    @type timeout: int/float
+    @param timeout: Timeout in seconds (values larger than L{WFJC_TIMEOUT} will
+                    be capped to that value)
+
+    """
+    assert timeout >= 0, "Timeout can not be negative"
     return self.CallMethod(REQ_WAIT_FOR_JOB_CHANGE,
                            (job_id, fields, prev_job_info,
-                            prev_log_serial, timeout))
+                            prev_log_serial,
+                            min(WFJC_TIMEOUT, timeout)))
 
   def WaitForJobChange(self, job_id, fields, prev_job_info, prev_log_serial):
     while True:
