@@ -52,8 +52,9 @@ class BaseOpCode(object):
     __slots__ attribute for this class.
 
     """
+    slots = self._all_slots()
     for key in kwargs:
-      if key not in self.__slots__:
+      if key not in slots:
         raise TypeError("Object %s doesn't support the parameter '%s'" %
                         (self.__class__.__name__, key))
       setattr(self, key, kwargs[key])
@@ -69,7 +70,7 @@ class BaseOpCode(object):
 
     """
     state = {}
-    for name in self.__slots__:
+    for name in self._all_slots():
       if hasattr(self, name):
         state[name] = getattr(self, name)
     return state
@@ -88,13 +89,23 @@ class BaseOpCode(object):
       raise ValueError("Invalid data to __setstate__: expected dict, got %s" %
                        type(state))
 
-    for name in self.__slots__:
-      if name not in state:
+    for name in self._all_slots():
+      if name not in state and hasattr(self, name):
         delattr(self, name)
 
     for name in state:
       setattr(self, name, state[name])
 
+  @classmethod
+  def _all_slots(cls):
+    """Compute the list of all declared slots for a class.
+
+    """
+    slots = []
+    for parent in cls.__mro__:
+      slots.extend(getattr(parent, "__slots__", []))
+    return slots
+
 
 class OpCode(BaseOpCode):
   """Abstract OpCode.
@@ -103,11 +114,13 @@ class OpCode(BaseOpCode):
   from this class should override OP_ID.
 
   @cvar OP_ID: The ID of this opcode. This should be unique amongst all
-               childre of this class.
+               children of this class.
+  @ivar dry_run: Whether the LU should be run in dry-run mode, i.e. just
+                 the check steps
 
   """
   OP_ID = "OP_ABSTRACT"
-  __slots__ = []
+  __slots__ = ["dry_run", "debug_level"]
 
   def __getstate__(self):
     """Specialized getstate for opcodes.
@@ -168,6 +181,17 @@ class OpCode(BaseOpCode):
 
 # cluster opcodes
 
+class OpPostInitCluster(OpCode):
+  """Post cluster initialization.
+
+  This opcode does not touch the cluster at all. Its purpose is to run hooks
+  after the cluster has been initialized.
+
+  """
+  OP_ID = "OP_CLUSTER_POST_INIT"
+  __slots__ = []
+
+
 class OpDestroyCluster(OpCode):
   """Destroy the cluster.
 
@@ -196,7 +220,8 @@ class OpVerifyCluster(OpCode):
 
   """
   OP_ID = "OP_CLUSTER_VERIFY"
-  __slots__ = ["skip_checks"]
+  __slots__ = ["skip_checks", "verbose", "error_codes",
+               "debug_simulate_errors"]
 
 
 class OpVerifyDisks(OpCode):
@@ -276,8 +301,14 @@ class OpSetClusterParams(OpCode):
     "vg_name",
     "enabled_hypervisors",
     "hvparams",
+    "os_hvp",
     "beparams",
+    "nicparams",
     "candidate_pool_size",
+    "maintain_node_health",
+    "uid_pool",
+    "add_uids",
+    "remove_uids",
     ]
 
 
@@ -286,8 +317,7 @@ class OpRedistributeConfig(OpCode):
 
   """
   OP_ID = "OP_CLUSTER_REDIST_CONF"
-  __slots__ = [
-    ]
+  __slots__ = []
 
 # node opcodes
 
@@ -343,6 +373,40 @@ class OpQueryNodeVolumes(OpCode):
   __slots__ = ["nodes", "output_fields"]
 
 
+class OpQueryNodeStorage(OpCode):
+  """Get information on storage for node(s)."""
+  OP_ID = "OP_NODE_QUERY_STORAGE"
+  __slots__ = [
+    "nodes",
+    "storage_type",
+    "name",
+    "output_fields",
+    ]
+
+
+class OpModifyNodeStorage(OpCode):
+  """Modifies the properies of a storage unit"""
+  OP_ID = "OP_NODE_MODIFY_STORAGE"
+  __slots__ = [
+    "node_name",
+    "storage_type",
+    "name",
+    "changes",
+    ]
+
+
+class OpRepairNodeStorage(OpCode):
+  """Repairs the volume group on a node."""
+  OP_ID = "OP_REPAIR_NODE_STORAGE"
+  OP_DSC_FIELD = "node_name"
+  __slots__ = [
+    "node_name",
+    "storage_type",
+    "name",
+    "ignore_consistency",
+    ]
+
+
 class OpSetNodeParams(OpCode):
   """Change the parameters of a node."""
   OP_ID = "OP_NODE_SET_PARAMS"
@@ -353,8 +417,46 @@ class OpSetNodeParams(OpCode):
     "master_candidate",
     "offline",
     "drained",
+    "auto_promote",
     ]
 
+
+class OpPowercycleNode(OpCode):
+  """Tries to powercycle a node."""
+  OP_ID = "OP_NODE_POWERCYCLE"
+  OP_DSC_FIELD = "node_name"
+  __slots__ = [
+    "node_name",
+    "force",
+    ]
+
+
+class OpEvacuateNode(OpCode):
+  """Relocate secondary instances from a node."""
+  OP_ID = "OP_NODE_EVACUATE"
+  OP_DSC_FIELD = "node_name"
+  __slots__ = [
+    "node_name", "remote_node", "iallocator", "early_release",
+    ]
+
+
+class OpMigrateNode(OpCode):
+  """Migrate all instances from a node."""
+  OP_ID = "OP_NODE_MIGRATE"
+  OP_DSC_FIELD = "node_name"
+  __slots__ = [
+    "node_name",
+    "live",
+    ]
+
+
+class OpNodeEvacuationStrategy(OpCode):
+  """Compute the evacuation strategy for a list of nodes."""
+  OP_ID = "OP_NODE_EVAC_STRATEGY"
+  OP_DSC_FIELD = "nodes"
+  __slots__ = ["nodes", "iallocator", "remote_node"]
+
+
 # instance opcodes
 
 class OpCreateInstance(OpCode):
@@ -362,14 +464,16 @@ class OpCreateInstance(OpCode):
   OP_ID = "OP_INSTANCE_CREATE"
   OP_DSC_FIELD = "instance_name"
   __slots__ = [
-    "instance_name", "os_type", "pnode",
-    "disk_template", "snode", "mode",
+    "instance_name",
+    "os_type", "force_variant", "no_install",
+    "pnode", "disk_template", "snode", "mode",
     "disks", "nics",
-    "src_node", "src_path", "start",
-    "wait_for_sync", "ip_check",
+    "src_node", "src_path", "start", "identify_defaults",
+    "wait_for_sync", "ip_check", "name_check",
     "file_storage_dir", "file_driver",
     "iallocator",
     "hypervisor", "hvparams", "beparams",
+    "dry_run",
     ]
 
 
@@ -377,55 +481,70 @@ class OpReinstallInstance(OpCode):
   """Reinstall an instance's OS."""
   OP_ID = "OP_INSTANCE_REINSTALL"
   OP_DSC_FIELD = "instance_name"
-  __slots__ = ["instance_name", "os_type"]
+  __slots__ = ["instance_name", "os_type", "force_variant"]
 
 
 class OpRemoveInstance(OpCode):
   """Remove an instance."""
   OP_ID = "OP_INSTANCE_REMOVE"
   OP_DSC_FIELD = "instance_name"
-  __slots__ = ["instance_name", "ignore_failures"]
+  __slots__ = [
+    "instance_name",
+    "ignore_failures",
+    "shutdown_timeout",
+    ]
 
 
 class OpRenameInstance(OpCode):
   """Rename an instance."""
   OP_ID = "OP_INSTANCE_RENAME"
-  __slots__ = ["instance_name", "ignore_ip", "new_name"]
+  __slots__ = [
+    "instance_name", "ignore_ip", "new_name",
+    ]
 
 
 class OpStartupInstance(OpCode):
   """Startup an instance."""
   OP_ID = "OP_INSTANCE_STARTUP"
   OP_DSC_FIELD = "instance_name"
-  __slots__ = ["instance_name", "force", "hvparams", "beparams"]
+  __slots__ = [
+    "instance_name", "force", "hvparams", "beparams",
+    ]
 
 
 class OpShutdownInstance(OpCode):
   """Shutdown an instance."""
   OP_ID = "OP_INSTANCE_SHUTDOWN"
   OP_DSC_FIELD = "instance_name"
-  __slots__ = ["instance_name"]
+  __slots__ = ["instance_name", "timeout"]
 
 
 class OpRebootInstance(OpCode):
   """Reboot an instance."""
   OP_ID = "OP_INSTANCE_REBOOT"
   OP_DSC_FIELD = "instance_name"
-  __slots__ = ["instance_name", "reboot_type", "ignore_secondaries" ]
+  __slots__ = [
+    "instance_name", "reboot_type", "ignore_secondaries", "shutdown_timeout",
+    ]
 
 
 class OpReplaceDisks(OpCode):
   """Replace the disks of an instance."""
   OP_ID = "OP_INSTANCE_REPLACE_DISKS"
   OP_DSC_FIELD = "instance_name"
-  __slots__ = ["instance_name", "remote_node", "mode", "disks", "iallocator"]
+  __slots__ = [
+    "instance_name", "remote_node", "mode", "disks", "iallocator",
+    "early_release",
+    ]
 
 
 class OpFailoverInstance(OpCode):
   """Failover an instance."""
   OP_ID = "OP_INSTANCE_FAILOVER"
   OP_DSC_FIELD = "instance_name"
-  __slots__ = ["instance_name", "ignore_consistency"]
+  __slots__ = [
+    "instance_name", "ignore_consistency", "shutdown_timeout",
+    ]
 
 
 class OpMigrateInstance(OpCode):
@@ -442,6 +561,23 @@ class OpMigrateInstance(OpCode):
   __slots__ = ["instance_name", "live", "cleanup"]
 
 
+class OpMoveInstance(OpCode):
+  """Move an instance.
+
+  This move (with shutting down an instance and data copying) to an
+  arbitrary node.
+
+  @ivar instance_name: the name of the instance
+  @ivar target_node: the destination node
+
+  """
+  OP_ID = "OP_INSTANCE_MOVE"
+  OP_DSC_FIELD = "instance_name"
+  __slots__ = [
+    "instance_name", "target_node", "shutdown_timeout",
+    ]
+
+
 class OpConnectConsole(OpCode):
   """Connect to an instance's console."""
   OP_ID = "OP_INSTANCE_CONSOLE"
@@ -463,6 +599,13 @@ class OpDeactivateInstanceDisks(OpCode):
   __slots__ = ["instance_name"]
 
 
+class OpRecreateInstanceDisks(OpCode):
+  """Deactivate an instance's disks."""
+  OP_ID = "OP_INSTANCE_RECREATE_DISKS"
+  OP_DSC_FIELD = "instance_name"
+  __slots__ = ["instance_name", "disks"]
+
+
 class OpQueryInstances(OpCode):
   """Compute the list of instances."""
   OP_ID = "OP_INSTANCE_QUERY"
@@ -482,7 +625,8 @@ class OpSetInstanceParams(OpCode):
   __slots__ = [
     "instance_name",
     "hvparams", "beparams", "force",
-    "nics", "disks",
+    "nics", "disks", "disk_template",
+    "remote_node", "os_name", "force_variant",
     ]
 
 
@@ -490,7 +634,9 @@ class OpGrowDisk(OpCode):
   """Grow a disk of an instance."""
   OP_ID = "OP_INSTANCE_GROW_DISK"
   OP_DSC_FIELD = "instance_name"
-  __slots__ = ["instance_name", "disk", "amount", "wait_for_sync"]
+  __slots__ = [
+    "instance_name", "disk", "amount", "wait_for_sync",
+    ]
 
 
 # OS opcodes
@@ -511,7 +657,9 @@ class OpExportInstance(OpCode):
   """Export an instance."""
   OP_ID = "OP_BACKUP_EXPORT"
   OP_DSC_FIELD = "instance_name"
-  __slots__ = ["instance_name", "target_node", "shutdown"]
+  __slots__ = [
+    "instance_name", "target_node", "shutdown", "shutdown_timeout",
+    ]
 
 
 class OpRemoveExport(OpCode):
@@ -592,8 +740,10 @@ class OpTestAllocator(OpCode):
     "direction", "mode", "allocator", "name",
     "mem_size", "disks", "disk_template",
     "os", "tags", "nics", "vcpus", "hypervisor",
+    "evac_nodes",
     ]
 
+
 OP_MAPPING = dict([(v.OP_ID, v) for v in globals().values()
                    if (isinstance(v, type) and issubclass(v, OpCode) and
                        hasattr(v, "OP_ID"))])
