@@ -45,6 +45,7 @@ from ganeti import opcodes
 from ganeti import http
 from ganeti import constants
 from ganeti import cli
+from ganeti import utils
 from ganeti import rapi
 from ganeti.rapi import baserlib
 
@@ -83,6 +84,15 @@ _NR_MAP = {
   "R": _NR_REGULAR,
   }
 
+# Request data version field
+_REQ_DATA_VERSION = "__version__"
+
+# Feature string for instance creation request data version 1
+_INST_CREATE_REQV1 = "instance-create-reqv1"
+
+# Timeout for /2/jobs/[job_id]/wait. Gives job up to 10 seconds to change.
+_WFJC_TIMEOUT = 10
+
 
 class R_version(baserlib.R_Generic):
   """/version resource.
@@ -112,6 +122,18 @@ class R_2_info(baserlib.R_Generic):
     return client.QueryClusterInfo()
 
 
+class R_2_features(baserlib.R_Generic):
+  """/2/features resource.
+
+  """
+  @staticmethod
+  def GET():
+    """Returns list of optional RAPI features implemented.
+
+    """
+    return [_INST_CREATE_REQV1]
+
+
 class R_2_os(baserlib.R_Generic):
   """/2/os resource.
 
@@ -211,6 +233,55 @@ class R_2_jobs_id(baserlib.R_Generic):
     return result
 
 
+class R_2_jobs_id_wait(baserlib.R_Generic):
+  """/2/jobs/[job_id]/wait resource.
+
+  """
+  # WaitForJobChange provides access to sensitive information and blocks
+  # machine resources (it's a blocking RAPI call), hence restricting access.
+  GET_ACCESS = [rapi.RAPI_ACCESS_WRITE]
+
+  def GET(self):
+    """Waits for job changes.
+
+    """
+    job_id = self.items[0]
+
+    fields = self.getBodyParameter("fields")
+    prev_job_info = self.getBodyParameter("previous_job_info", None)
+    prev_log_serial = self.getBodyParameter("previous_log_serial", None)
+
+    if not isinstance(fields, list):
+      raise http.HttpBadRequest("The 'fields' parameter should be a list")
+
+    if not (prev_job_info is None or isinstance(prev_job_info, list)):
+      raise http.HttpBadRequest("The 'previous_job_info' parameter should"
+                                " be a list")
+
+    if not (prev_log_serial is None or
+            isinstance(prev_log_serial, (int, long))):
+      raise http.HttpBadRequest("The 'previous_log_serial' parameter should"
+                                " be a number")
+
+    client = baserlib.GetClient()
+    result = client.WaitForJobChangeOnce(job_id, fields,
+                                         prev_job_info, prev_log_serial,
+                                         timeout=_WFJC_TIMEOUT)
+    if not result:
+      raise http.HttpNotFound()
+
+    if result == constants.JOB_NOTCHANGED:
+      # No changes
+      return None
+
+    (job_info, log_entries) = result
+
+    return {
+      "job_info": job_info,
+      "log_entries": log_entries,
+      }
+
+
 class R_2_nodes(baserlib.R_Generic):
   """/2/nodes resource.
 
@@ -241,8 +312,10 @@ class R_2_nodes_name(baserlib.R_Generic):
     """
     node_name = self.items[0]
     client = baserlib.GetClient()
-    result = client.QueryNodes(names=[node_name], fields=N_FIELDS,
-                               use_locking=self.useLocking())
+
+    result = baserlib.HandleItemQueryErrors(client.QueryNodes,
+                                            names=[node_name], fields=N_FIELDS,
+                                            use_locking=self.useLocking())
 
     return baserlib.MapFields(N_FIELDS, result[0])
 
@@ -419,6 +492,100 @@ class R_2_nodes_name_storage_repair(baserlib.R_Generic):
     return baserlib.SubmitJob([op])
 
 
+def _ParseInstanceCreateRequestVersion1(data, dry_run):
+  """Parses an instance creation request version 1.
+
+  @rtype: L{opcodes.OpCreateInstance}
+  @return: Instance creation opcode
+
+  """
+  # Disks
+  disks_input = baserlib.CheckParameter(data, "disks", exptype=list)
+
+  disks = []
+  for idx, i in enumerate(disks_input):
+    baserlib.CheckType(i, dict, "Disk %d specification" % idx)
+
+    # Size is mandatory
+    try:
+      size = i[constants.IDISK_SIZE]
+    except KeyError:
+      raise http.HttpBadRequest("Disk %d specification wrong: missing disk"
+                                " size" % idx)
+
+    disk = {
+      constants.IDISK_SIZE: size,
+      }
+
+    # Optional disk access mode
+    try:
+      disk_access = i[constants.IDISK_MODE]
+    except KeyError:
+      pass
+    else:
+      disk[constants.IDISK_MODE] = disk_access
+
+    disks.append(disk)
+
+  assert len(disks_input) == len(disks)
+
+  # Network interfaces
+  nics_input = baserlib.CheckParameter(data, "nics", exptype=list)
+
+  nics = []
+  for idx, i in enumerate(nics_input):
+    baserlib.CheckType(i, dict, "NIC %d specification" % idx)
+
+    nic = {}
+
+    for field in constants.INIC_PARAMS:
+      try:
+        value = i[field]
+      except KeyError:
+        continue
+
+      nic[field] = value
+
+    nics.append(nic)
+
+  assert len(nics_input) == len(nics)
+
+  # HV/BE parameters
+  hvparams = baserlib.CheckParameter(data, "hvparams", default={})
+  utils.ForceDictType(hvparams, constants.HVS_PARAMETER_TYPES)
+
+  beparams = baserlib.CheckParameter(data, "beparams", default={})
+  utils.ForceDictType(beparams, constants.BES_PARAMETER_TYPES)
+
+  return opcodes.OpCreateInstance(
+    mode=baserlib.CheckParameter(data, "mode"),
+    instance_name=baserlib.CheckParameter(data, "name"),
+    os_type=baserlib.CheckParameter(data, "os", default=None),
+    force_variant=baserlib.CheckParameter(data, "force_variant",
+                                          default=False),
+    pnode=baserlib.CheckParameter(data, "pnode", default=None),
+    snode=baserlib.CheckParameter(data, "snode", default=None),
+    disk_template=baserlib.CheckParameter(data, "disk_template"),
+    disks=disks,
+    nics=nics,
+    src_node=baserlib.CheckParameter(data, "src_node", default=None),
+    src_path=baserlib.CheckParameter(data, "src_path", default=None),
+    start=baserlib.CheckParameter(data, "start", default=True),
+    wait_for_sync=True,
+    ip_check=baserlib.CheckParameter(data, "ip_check", default=True),
+    name_check=baserlib.CheckParameter(data, "name_check", default=True),
+    file_storage_dir=baserlib.CheckParameter(data, "file_storage_dir",
+                                             default=None),
+    file_driver=baserlib.CheckParameter(data, "file_driver",
+                                        default=constants.FD_LOOP),
+    iallocator=baserlib.CheckParameter(data, "iallocator", default=None),
+    hypervisor=baserlib.CheckParameter(data, "hypervisor", default=None),
+    hvparams=hvparams,
+    beparams=beparams,
+    dry_run=dry_run,
+    )
+
+
 class R_2_instances(baserlib.R_Generic):
   """/2/instances resource.
 
@@ -439,15 +606,16 @@ class R_2_instances(baserlib.R_Generic):
       return baserlib.BuildUriList(instanceslist, "/2/instances/%s",
                                    uri_fields=("id", "uri"))
 
-  def POST(self):
-    """Create an instance.
+  def _ParseVersion0CreateRequest(self):
+    """Parses an instance creation request version 0.
 
-    @return: a job id
+    Request data version 0 is deprecated and should not be used anymore.
+
+    @rtype: L{opcodes.OpCreateInstance}
+    @return: Instance creation opcode
 
     """
-    if not isinstance(self.req.request_body, dict):
-      raise http.HttpBadRequest("Invalid body contents, not a dictionary")
-
+    # Do not modify anymore, request data version 0 is deprecated
     beparams = baserlib.MakeParamsDict(self.req.request_body,
                                        constants.BES_PARAMETERS)
     hvparams = baserlib.MakeParamsDict(self.req.request_body,
@@ -464,6 +632,7 @@ class R_2_instances(baserlib.R_Generic):
         raise http.HttpBadRequest("Disk %d specification wrong: should"
                                   " be an integer" % idx)
       disks.append({"size": d})
+
     # nic processing (one nic only)
     nics = [{"mac": fn("mac", constants.VALUE_AUTO)}]
     if fn("ip", None) is not None:
@@ -475,7 +644,8 @@ class R_2_instances(baserlib.R_Generic):
     if fn("bridge", None) is not None:
       nics[0]["bridge"] = fn("bridge")
 
-    op = opcodes.OpCreateInstance(
+    # Do not modify anymore, request data version 0 is deprecated
+    return opcodes.OpCreateInstance(
       mode=constants.INSTANCE_CREATE,
       instance_name=fn('name'),
       disks=disks,
@@ -493,10 +663,31 @@ class R_2_instances(baserlib.R_Generic):
       hvparams=hvparams,
       beparams=beparams,
       file_storage_dir=fn('file_storage_dir', None),
-      file_driver=fn('file_driver', 'loop'),
+      file_driver=fn('file_driver', constants.FD_LOOP),
       dry_run=bool(self.dryRun()),
       )
 
+  def POST(self):
+    """Create an instance.
+
+    @return: a job id
+
+    """
+    if not isinstance(self.req.request_body, dict):
+      raise http.HttpBadRequest("Invalid body contents, not a dictionary")
+
+    # Default to request data version 0
+    data_version = self.getBodyParameter(_REQ_DATA_VERSION, 0)
+
+    if data_version == 0:
+      op = self._ParseVersion0CreateRequest()
+    elif data_version == 1:
+      op = _ParseInstanceCreateRequestVersion1(self.req.request_body,
+                                               self.dryRun())
+    else:
+      raise http.HttpBadRequest("Unsupported request data version %s" %
+                                data_version)
+
     return baserlib.SubmitJob([op])
 
 
@@ -510,8 +701,11 @@ class R_2_instances_name(baserlib.R_Generic):
     """
     client = baserlib.GetClient()
     instance_name = self.items[0]
-    result = client.QueryInstances(names=[instance_name], fields=I_FIELDS,
-                                   use_locking=self.useLocking())
+
+    result = baserlib.HandleItemQueryErrors(client.QueryInstances,
+                                            names=[instance_name],
+                                            fields=I_FIELDS,
+                                            use_locking=self.useLocking())
 
     return baserlib.MapFields(I_FIELDS, result[0])
 
