@@ -281,10 +281,13 @@ def main():
 
   rapi_user = "ganeti-qa"
   rapi_secret = utils.GenerateSecret()
-  qa_rapi.OpenerFactory.SetCredentials(rapi_user, rapi_secret)
 
   RunEnvTests()
   SetupCluster(rapi_user, rapi_secret)
+
+  # Load RAPI certificate
+  qa_rapi.Setup(rapi_user, rapi_secret)
+
   RunClusterTests()
   RunOsTests()
 
@@ -308,10 +311,12 @@ def main():
       RunTest(qa_rapi.TestNode, pnode)
 
       if qa_config.TestEnabled("instance-add-plain-disk"):
-        rapi_instance = RunTest(qa_rapi.TestRapiInstanceAdd, pnode)
-        RunCommonInstanceTests(rapi_instance)
-        RunTest(qa_rapi.TestRapiInstanceRemove, rapi_instance)
-        del rapi_instance
+        for use_client in [True, False]:
+          rapi_instance = RunTest(qa_rapi.TestRapiInstanceAdd, pnode,
+                                  use_client)
+          RunCommonInstanceTests(rapi_instance)
+          RunTest(qa_rapi.TestRapiInstanceRemove, rapi_instance, use_client)
+          del rapi_instance
 
     if qa_config.TestEnabled('instance-add-plain-disk'):
       instance = RunTest(qa_instance.TestInstanceAddWithPlainDisk, pnode)
@@ -333,7 +338,9 @@ def main():
           instance = RunTest(func, pnode, snode)
           RunCommonInstanceTests(instance)
           if qa_config.TestEnabled('instance-convert-disk'):
+            RunTest(qa_instance.TestInstanceShutdown, instance)
             RunTest(qa_instance.TestInstanceConvertDisk, instance, snode)
+            RunTest(qa_instance.TestInstanceStartup, instance)
           RunExportImportTests(instance, pnode)
           RunHardwareFailureTests(instance, pnode, snode)
           RunTest(qa_instance.TestInstanceRemove, instance)
