@@ -182,32 +182,45 @@ def TestClusterRenewCrypto():
   AssertNotEqual(StartSSH(master["primary"],
                           utils.ShellQuoteArgs(cmd)).wait(), 0)
 
-  # Custom RAPI certificate
-  fh = tempfile.NamedTemporaryFile()
-
-  # Ensure certificate doesn't cause "gnt-cluster verify" to complain
-  validity = constants.SSL_CERT_EXPIRATION_WARN * 3
-
-  bootstrap.GenerateSelfSignedSslCert(fh.name, validity=validity)
-
-  tmpcert = qa_utils.UploadFile(master["primary"], fh.name)
+  rapi_cert_backup = qa_utils.BackupFile(master["primary"],
+                                         constants.RAPI_CERT_FILE)
   try:
+    # Custom RAPI certificate
+    fh = tempfile.NamedTemporaryFile()
+
+    # Ensure certificate doesn't cause "gnt-cluster verify" to complain
+    validity = constants.SSL_CERT_EXPIRATION_WARN * 3
+
+    bootstrap.GenerateSelfSignedSslCert(fh.name, validity=validity)
+
+    tmpcert = qa_utils.UploadFile(master["primary"], fh.name)
+    try:
+      cmd = ["gnt-cluster", "renew-crypto", "--force",
+             "--rapi-certificate=%s" % tmpcert]
+      AssertEqual(StartSSH(master["primary"],
+                           utils.ShellQuoteArgs(cmd)).wait(), 0)
+    finally:
+      cmd = ["rm", "-f", tmpcert]
+      AssertEqual(StartSSH(master["primary"],
+                           utils.ShellQuoteArgs(cmd)).wait(), 0)
+
+    # Normal case
     cmd = ["gnt-cluster", "renew-crypto", "--force",
-           "--rapi-certificate=%s" % tmpcert]
+           "--new-cluster-certificate", "--new-confd-hmac-key",
+           "--new-rapi-certificate"]
+    AssertEqual(StartSSH(master["primary"],
+                         utils.ShellQuoteArgs(cmd)).wait(), 0)
+
+    # Restore RAPI certificate
+    cmd = ["gnt-cluster", "renew-crypto", "--force",
+           "--rapi-certificate=%s" % rapi_cert_backup]
     AssertEqual(StartSSH(master["primary"],
                          utils.ShellQuoteArgs(cmd)).wait(), 0)
   finally:
-    cmd = ["rm", "-f", tmpcert]
+    cmd = ["rm", "-f", rapi_cert_backup]
     AssertEqual(StartSSH(master["primary"],
                          utils.ShellQuoteArgs(cmd)).wait(), 0)
 
-  # Normal case
-  cmd = ["gnt-cluster", "renew-crypto", "--force",
-         "--new-cluster-certificate", "--new-confd-hmac-key",
-         "--new-rapi-certificate"]
-  AssertEqual(StartSSH(master["primary"],
-                       utils.ShellQuoteArgs(cmd)).wait(), 0)
-
 
 def TestClusterBurnin():
   """Burnin"""
@@ -218,6 +231,8 @@ def TestClusterBurnin():
   parallel = options.get('burnin-in-parallel', False)
   check_inst = options.get('burnin-check-instances', False)
   do_rename = options.get('burnin-rename', '')
+  do_reboot = options.get('burnin-reboot', True)
+  reboot_types = options.get("reboot-types", constants.REBOOT_TYPES)
 
   # Get as many instances as we need
   instances = []
@@ -247,6 +262,10 @@ def TestClusterBurnin():
         cmd.append('--http-check')
       if do_rename:
         cmd.append('--rename=%s' % do_rename)
+      if not do_reboot:
+        cmd.append('--no-reboot')
+      else:
+        cmd.append('--reboot-types=%s' % ",".join(reboot_types))
       cmd += [inst['name'] for inst in instances]
       AssertEqual(StartSSH(master['primary'],
                            utils.ShellQuoteArgs(cmd)).wait(), 0)
