@@ -111,9 +111,11 @@ def TestInstanceReboot(instance):
   """gnt-instance reboot"""
   master = qa_config.GetMasterNode()
 
-  for reboottype in ["soft", "hard", "full"]:
-    cmd = ['gnt-instance', 'reboot', '--type=%s' % reboottype,
-           instance['name']]
+  options = qa_config.get('options', {})
+  reboot_types = options.get("reboot-types", constants.REBOOT_TYPES)
+
+  for rtype in reboot_types:
+    cmd = ['gnt-instance', 'reboot', '--type=%s' % rtype, instance['name']]
     AssertEqual(StartSSH(master['primary'],
                          utils.ShellQuoteArgs(cmd)).wait(), 0)
 
@@ -215,7 +217,8 @@ def TestInstanceConvertDisk(instance, snode):
   cmd = ['gnt-instance', 'modify', '-t', 'plain', instance['name']]
   AssertEqual(StartSSH(master['primary'],
                        utils.ShellQuoteArgs(cmd)).wait(), 0)
-  cmd = ['gnt-instance', 'modify', '-t', 'drbd', '-n', snode, instance['name']]
+  cmd = ['gnt-instance', 'modify', '-t', 'drbd', '-n', snode['primary'],
+         instance['name']]
   AssertEqual(StartSSH(master['primary'],
                        utils.ShellQuoteArgs(cmd)).wait(), 0)
 
