@@ -25,10 +25,11 @@
 import os
 import unittest
 
-import testutils
 from ganeti import bdev
 from ganeti import errors
 
+import testutils
+
 
 class TestDRBD8Runner(testutils.GanetiTestCase):
   """Testing case for DRBD8"""
@@ -189,4 +190,4 @@ class TestDRBD8Status(testutils.GanetiTestCase):
                       stats.is_disk_uptodate)
 
 if __name__ == '__main__':
-  unittest.main()
+  testutils.GanetiTestProgram()
